@@ -11,10 +11,12 @@ or by contacting Anthony Marquez at amarquez@ap.org.
 """
 import os
 import csv
+import copy
 import itertools
 import calculate
 from ftplib import FTP
 from datetime import date
+from pprint import pprint
 from cStringIO import StringIO
 from BeautifulSoup import BeautifulStoneSoup
 from dateutil.parser import parse as dateparse
@@ -81,16 +83,22 @@ class AP(object):
         self.ftp.quit()
         return results
     
-    def get_topofticket(self, election_date, **kwargs):
+    def get_topofticket(self, election_date=None, **kwargs):
         """
         Takes a date in any common format (YYYY-MM-DD is preferred) 
         and returns the results for that date.
+        
+        If you do not provide a date, it defaults to the next major election.
+        Today that is the Nov. 6, 2012 general election.
         """
-        try:
-            dt = dateparse(election_date)
-        except ValueError:
-            raise ValueError("The election date you've submitted could not be parsed. Try submitting it in YYYY-MM-DD format.")
-        result = TopOfTicket(self, dt.strftime("%Y%m%d"), **kwargs)
+        if election_date:
+            try:
+                dt = dateparse(election_date)
+            except ValueError:
+                raise ValueError("The election date you've submitted could not be parsed. Try submitting it in YYYY-MM-DD format.")
+            result = TopOfTicket(self, dt.strftime("%Y%m%d"), **kwargs)
+        else:
+            result = TopOfTicket(self, **kwargs)
         self.ftp.quit()
         return result
     
@@ -101,6 +109,39 @@ class AP(object):
         nomination contest held by the two major parties.
         """
         return DelegateSummary(self).nominations
+    
+    def get_presidential_summary(self, districts=False):
+        """
+        Returns a summary of presidential election results at three levels:
+        
+            1. Nationwide popular vote and electoral vote
+            2. State-level popular vote and electoral vote
+            3. County-level popular vote
+        
+        If `districts` is provided and set to True the results will include 
+        Congressional district-level results in the two states that break out
+        their presidential electors: Maine and Nebraska. This feature only works
+        if the AP has given your account access to the ME and NE data folders.
+        
+        By default, `districts` is set to False.
+        """
+        result = PresidentialSummary(self, districts=districts)
+        self.ftp.quit()
+        return result
+
+    def get_congressional_trends(self, results=True):
+        """
+        Returns a summary of the current balance of power in 
+        each chamber of congress.
+        
+        Usage: 
+            >>> trends = client.get_congressional_trends()
+            >>> trends.house.gop_net_change
+            -2
+        """
+        result = CongressionalTrends(self, results)
+        self.ftp.quit()
+        return result
 
     #
     # Private methods
@@ -393,7 +434,10 @@ class BaseAPResultCollection(object):
                 first_name = cand['pol_first_name'],
                 middle_name = cand['pol_middle_name'],
                 last_name = cand['pol_last_name'],
-                ap_race_number = cand['ra_number'],
+                ap_race_number = self.ap_number_template % ({
+                    'number': cand['ra_number'], 
+                    'state': cand.get('st_postal', 'None')
+                }),
                 ap_natl_number = cand['pol_nat_id'],
                 ap_polra_number = cand['polra_number'],
                 ap_pol_number = cand['pol_number'],
@@ -402,7 +446,10 @@ class BaseAPResultCollection(object):
                 party = cand['polra_party'],
                 # use_suffix?
             )
-            self._races[candidate.ap_race_number].add_candidate(candidate)
+            try:
+                self._races[candidate.ap_race_number].add_candidate(candidate)
+            except KeyError:
+                pass
     
     def _init_races(self):
         """
@@ -413,8 +460,11 @@ class BaseAPResultCollection(object):
         # Loop through it all
         for race in race_list:
             # Create a Race object...
-            race = Race(
-                ap_race_number = race['ra_number'],
+            race = Race( # self.name
+                ap_race_number = self.ap_number_template % ({
+                    'number': race['ra_number'], 
+                    'state': race.get('st_postal', 'None')
+                }),
                 office_name = race['ot_name'],
                 office_description = race['of_description'],
                 office_id = race['office_id'],
@@ -578,14 +628,21 @@ class BaseAPResultCollection(object):
         # Start looping through the lines...
         for row in flat_list:
             
-            # Get the race
-            race = self.get_race(row['race_number'])
+            # Get the race, with a special case for the presidential race
+            ap_race_number = self.ap_number_template % ({
+                'number': row['race_number'], 
+                'state': row['state_postal']
+            })
+            try:
+                race = self.get_race(ap_race_number)
+            except KeyError:
+                continue
             
             # Figure out if it's a state or a county
             fips =row['fips']
             is_state = row['county_number'] == '1'
             county_number = str(row['county_number'])
-            
+
             # Pull the reporting unit
             reporting_unit = race.get_reporting_unit(
                 "%s%s" % (row['county_name'], county_number)
@@ -600,6 +657,8 @@ class BaseAPResultCollection(object):
                 
                 # Pull the existing candidate object
                 candidate = race.get_candidate(cand["candidate_number"])
+                if not candidate:
+                    continue
                 
                 # Pull the vote total
                 vote_count = int(cand['vote_count'])
@@ -659,8 +718,9 @@ class State(BaseAPResultCollection):
     call. Contains, among its attributes, the results for all races recorded
     by the AP.
     """
-
-    def __init__(self, client, name, results=True, delegates=True):
+    ap_number_template = '%(number)s'
+    
+    def __init__(self, client, name, results=True, delegates=False):
         self.results_file_path = "/%(name)s/flat/%(name)s.txt" % {'name': name}
         self.delegates_file_path = "/%(name)s/flat/%(name)s_D.txt" % {'name': name}
         self.race_file_path = "/inits/%(name)s/%(name)s_race.txt" % {'name': name}
@@ -677,12 +737,22 @@ class TopOfTicket(BaseAPResultCollection):
     call. Contains, among its attributes, the results for all races recorded
     by the AP.
     """
-    def __init__(self, client, name, results=True, delegates=True):
-        self.results_file_path = "/Delegate_Tracking/US/flat/US_%(name)s.txt" % {'name': name}
-        self.delegates_file_path = "/Delegate_Tracking/US/flat/US_%(name)s_d.txt" % {'name': name}
-        self.race_file_path = "/inits/US/US_%(name)s_race.txt" % {'name': name}
-        self.reporting_unit_file_path = "/inits/US/US_%(name)s_ru.txt" % {'name': name}
-        self.candidate_file_path = "/inits/US/US_%(name)s_pol.txt" % {'name': name}
+    ap_number_template = '%(number)s-%(state)s'
+
+    def __init__(self, client, name='20121106', results=True, delegates=False):
+        # This means you're in a primary election with delegates at stake
+        if name and name != '20121106':
+            self.results_file_path = "/Delegate_Tracking/US/flat/US_%(name)s.txt" % {'name': name}
+            self.delegates_file_path = "/Delegate_Tracking/US/flat/US_%(name)s_d.txt" % {'name': name}
+            self.race_file_path = "/inits/US/US_%(name)s_race.txt" % {'name': name}
+            self.reporting_unit_file_path = "/inits/US/US_%(name)s_ru.txt" % {'name': name}
+            self.candidate_file_path = "/inits/US/US_%(name)s_pol.txt" % {'name': name}
+        # Otherwise we're on to general election 2012 as the default
+        else:
+            self.results_file_path = "/US_topofticket/flat/US.txt"
+            self.race_file_path = "/inits/US/US_%(name)s_race.txt" % {'name': name}
+            self.reporting_unit_file_path = "/inits/US/US_%(name)s_ru.txt" % {'name': name}
+            self.candidate_file_path = "/inits/US/US_%(name)s_pol.txt" % {'name': name}
         super(TopOfTicket, self).__init__(client, name, results, delegates)
     
     @property
@@ -690,6 +760,420 @@ class TopOfTicket(BaseAPResultCollection):
         return [o for o in self._reporting_units.values() if o.is_state]
 
 
+class PresidentialSummary(BaseAPResultCollection):
+    """
+    A summary of presidential election results, both popular and electoral vote,
+    at three levels:
+        
+        1. Nationwide
+        2. State-level
+        3  County-level
+    
+    Returned by the public get_presidential_summary method.
+    """
+    ap_number_template = '%(number)s-%(state)s'
+    
+    def __init__(self, client, districts=False):
+        self.results_file_path = "/Pres_Reports/flat/pres_county.txt"
+        self.race_file_path = "/inits/US/US_20121106_race.txt"
+        self.reporting_unit_file_path = "/inits/US/US_20121106_ru.txt"
+        self.candidate_file_path = "/inits/US/US_20121106_pol.txt"
+        self.summary_file_path = "/Pres_Reports/flat/pres_electoral.txt"
+        self.district_file_path = "/inits/US/US_20121106_district.txt"
+        self.fetch_districts = districts
+        super(PresidentialSummary, self).__init__(client, None, results=True, delegates=False)
+    
+    def _init_races(self):
+        """
+        Override the standard race init since there's only one race we want here.
+        """
+        # Do it normal...
+        super(PresidentialSummary, self)._init_races()
+        # Then get rid of everything that isn't the presidential race
+        self._races = dict((k,v) for k,v in self._races.items()
+            if v.office_name == 'President')
+        if self.fetch_districts:
+            # Init fake district race objects for the Nebraska...
+            new_neb = copy.deepcopy(self.filter_races(state_postal='NE')[0])
+            new_neb.scope = 'D'
+            new_neb.ap_race_number = self.ap_number_template % ({
+                'number': 'district',
+                'state': 'NE',
+            })
+            self._races.update({new_neb.ap_race_number: new_neb})
+            # .. and for Maine
+            new_maine = copy.deepcopy(self.filter_races(state_postal='ME')[0])
+            new_maine.scope = 'D'
+            new_maine.ap_race_number = self.ap_number_template % ({
+                'number': 'district',
+                'state': 'ME',
+            })
+            self._races.update({new_maine.ap_race_number: new_maine})
+    
+    def _init_reporting_units(self):
+        super(PresidentialSummary, self)._init_reporting_units()
+        if self.fetch_districts:
+            # Load the district file
+            d_list = self.client._fetch_csv(
+                self.district_file_path,
+                delimiter="|"
+            )
+            # Clear our any reporting units the races may already have
+            [setattr(r, '_reporting_units', {}) for r in self.filter_races(scope='D')]
+            # Loop through the district file
+            for r in d_list:
+                # Make a reporting unit for each
+                ru = ReportingUnit(
+                    name = "%s %s" % (r['st_postal'], r['dist_name']),
+                    ap_number = r['re_number'],
+                    abbrev = r['dist_number'],
+                    precincts_total = int(r['dist_precincts'])
+                )
+                # And connect it to the race
+                this_race = self.filter_races(
+                    scope='D',
+                    state_postal=r['st_postal']
+                )[0]
+                this_race._reporting_units.update({ru.key: ru})
+    
+    def fetch_results(self):
+        """
+        This will fetch and fill out all of the results. If called again,
+        it will simply run through and update all of the results with 
+        the most fresh data from the AP.
+        """
+        # Run the normal results fetch
+        super(PresidentialSummary, self).fetch_results()
+        # Fetch the nationwide results and electoral votes from separate file
+        summary_list = self.client._fetch_csv(
+            self.summary_file_path,
+            delimiter=";",
+            fieldnames=[
+                'test',
+                'election_date',
+                'state_postal',
+                'candidate_number',
+                'national_politician_id',
+                'electoral_votes',
+                'vote_count',
+                'is_winner',
+                'precincts_reporting',
+                'total_precincts',
+                'last_name',
+                'total_electoral_votes',
+            ]
+        )
+        # Loop through each record
+        for row in summary_list:
+            # If it's a state row...
+            if row['state_postal'] != 'US':
+                # ... tack the electoral votes on the reporting unit ...
+                this_ru = self.filter_races(state_postal=row['state_postal'], scope='S')[0].state
+                this_ru.electoral_votes_total = int(row['total_electoral_votes'])
+                # ...dig out the results object with this candidate...
+                this_result = [i for i in this_ru.results
+                    if i.candidate.ap_natl_number == row['national_politician_id']][0]
+                # ...set the electoral vote attribute...
+                this_result.electoral_votes_total = int(row['electoral_votes'])
+                this_ru.update_result(this_result)
+                # ...set the ru back into our set.
+                self._reporting_units.update({this_ru.key: this_ru})
+            # If it's a nationwide row start filling in the US reporting unit
+            else:
+                # Grab the race object
+                this_race = self.filter_races(scope='N')[0]
+                # Fabricate a reporting unit if one doesn't already exist...
+                if this_race.reporting_units:
+                    this_ru = this_race.reporting_units[0]
+                else:
+                    this_ru = ReportingUnit(
+                        name = 'US',
+                        ap_number = '0',
+                        fips = '00',
+                        abbrev = 'US',
+                        precincts_total = int(row['total_precincts']),
+                        precincts_reporting = int(row['precincts_reporting']),
+                        precincts_reporting_percent = calculate.percentage(
+                            int(row['precincts_reporting']),
+                            int(row['total_precincts']),
+                        ),
+                        electoral_votes_total = 538,
+                    )
+                # Grab the candidate object for this row
+                this_cand = [i for i in this_race.candidates
+                    if i.ap_natl_number == row['national_politician_id']][0]
+                # Set some extra attributes
+                this_cand.is_winner = row['is_winner'] == 'X'
+                this_cand.is_runoff = row['is_winner'] == 'R'
+                this_cand.is_incumbent = row['last_name'] == 'Obama'
+                # Create a new results object and connect it to the ru and cand
+                this_result = Result(
+                    candidate = this_cand,
+                    vote_total = int(row['vote_count']),
+                    reporting_unit = this_ru,
+                    electoral_votes_total = int(row['electoral_votes'])
+                )
+                this_ru.update_result(this_result)
+                this_race._reporting_units.update({this_ru.key: this_ru})
+                self._reporting_units.update({this_ru.key: this_ru})
+        
+        # Set the overall vote total and candidate vote percentages in nationwide
+        ru = self.nationwide
+        ru.votes_cast = sum([i.vote_total for i in ru.results])
+        for result in ru.results:
+            result.vote_total_percent  = calculate.percentage(
+                result.vote_total,
+                ru.votes_cast
+            )
+            ru.update_result(result)
+        self._reporting_units.update({ru.key: ru})
+        
+        if self.fetch_districts:
+            # Get the results in the Nebraska and Maine districts
+            district_result_paths = [
+                '/ME/flat/ME_D.txt',
+                '/NE/flat/NE_D.txt'
+            ]
+            for path in district_result_paths:
+                # Pull the data
+                flat_list = self.client._fetch_flatfile(
+                    path,
+                    [ # First the basic fields that will the same in each row
+                        'test',
+                        'election_date',
+                        'state_postal',
+                        'district_type',
+                        'district_number',
+                        'district_name',
+                        'race_number',
+                        'office_id',
+                        'race_type_id',
+                        'seat_number',
+                        'office_name',
+                        'seat_name',
+                        'race_type_party',
+                        'race_type',
+                        'office_description',
+                        'number_of_winners',
+                        'number_in_runoff',
+                        'precincts_reporting',
+                        'total_precincts',
+                    ],
+                   [ # Then the candidate fields that will repeat after the basics
+                        'candidate_number',
+                        'order',
+                        'party',
+                        'first_name',
+                        'middle_name',
+                        'last_name',
+                        'junior',
+                        'use_junior',
+                        'incumbent',
+                        'total_electoral_votes',
+                        'vote_count',
+                        'is_winner',
+                        'national_politician_id',
+                    ]
+                )
+                # Exclude statewide results
+                flat_list = [r for r in flat_list if r['district_number'] != '1']
+                # Loop through district-level results
+                for row in flat_list:
+                    # Get the race object
+                    this_race = self.filter_races(
+                        scope='D',
+                        state_postal=row['state_postal']
+                    )[0]
+                    # Get the reporting unit for this district
+                    ru_name = '%s %s%s' % (
+                        row['state_postal'],
+                        row['district_name'],
+                        row['district_number']
+                    )
+                    this_ru = this_race.get_reporting_unit(ru_name)
+                    votes_cast = 0
+                    # Loop through all the candidates in this row
+                    for cand in row['candidates']:
+                        # Create the candidate object
+                        this_cand = Candidate(
+                            first_name = cand['first_name'],
+                            middle_name = cand['middle_name'],
+                            last_name = cand['last_name'],
+                            ap_race_number = self.ap_number_template % ({
+                                'number': 'district',
+                                'state': row['state_postal'],
+                            }),
+                            ap_natl_number = cand['national_politician_id'],
+                            ap_polra_number = cand['candidate_number'],
+                            ap_pol_number = cand['candidate_number'],
+                            suffix = cand['junior'],
+                            party = cand['party'],
+                            is_winner = cand['is_winner'] == 'X',
+                            is_runoff = cand['is_winner'] == 'R',
+                            is_incumbent = cand['last_name'] == 'Obama'
+                        )
+                        
+                        # Pull the vote total
+                        vote_count = int(cand['vote_count'])
+                        
+                        # Add it to the overall total
+                        votes_cast += vote_count
+                        
+                        # Add it to the Race list
+                        this_race.add_candidate(this_cand)
+                        
+                        # Create the Result object
+                        this_result = Result(
+                            candidate = this_cand,
+                            vote_total = int(cand['vote_count']),
+                            reporting_unit = this_ru,
+                            electoral_votes_total = int(cand['total_electoral_votes'])
+                        )
+                        this_ru.update_result(this_result)
+                    
+                    # Sync the result with the reporting unit
+                    this_race._reporting_units.update({this_ru.key: this_ru})
+                    
+                    # Update the reporting unit's precincts status
+                    this_ru.precincts_total = int(row['total_precincts'])
+                    this_ru.precincts_reporting = int(row['precincts_reporting'])
+                    this_ru.precincts_reporting_percent = calculate.percentage(
+                        this_ru.precincts_reporting,
+                        this_ru.precincts_total
+                    )
+                    
+                    # Update the total votes cast
+                    this_ru.votes_cast = votes_cast
+                    
+                    # Loop back through the results and set the percentages now
+                    # that we know the overall total
+                    for result in this_ru.results:
+                        result.vote_total_percent = calculate.percentage(
+                            result.vote_total, 
+                            votes_cast
+                        )
+
+    
+    @property
+    def nationwide(self):
+        """
+        Return only the nationwide reporting unit
+        """
+        return [i.reporting_units[0] for i in self.races if i.scope == 'N'][0]
+    
+    @property
+    def states(self):
+        """
+        Return only the state-level reporting units
+        """
+        state_list = [i.state for i in self.races if i.scope == 'S']
+        return sorted(state_list, key=lambda x: x.name)
+    
+    @property
+    def counties(self):
+        """
+        Return only the county-level reporting units
+        """
+        state_list = [i for i in self.races if i.scope == 'S']
+        state_list = sorted(state_list, key=lambda x: x.state.name)
+        nested_list = [i.counties for i in state_list]
+        county_list = [item for sublist in nested_list for item in sublist]
+        return county_list
+    
+    @property
+    def districts(self):
+        """
+        Return only congressional-district results for states that split
+        out electoral votes, which is limited to Maine and Nebraska
+        """
+        race_list = self.filter_races(scope='D')
+        nested_list = [i.reporting_units for i in race_list]
+        district_list = [item for sublist in nested_list for item in sublist]
+        return district_list
+
+
+class CongressionalTrends(object):
+    """
+    A result collection that holds the two chambers of congress
+    and the current net changes for each party in each chamber.
+    """
+    def __init__(self, client, results=True):
+        self.client = client
+        self.chambers = []
+        if results:
+            self.chambers = self.get_trends()
+
+    def _process_trends(self, xml, fname):
+        soup = BeautifulStoneSoup(xml, selfClosingTags=['trend',])
+        name = soup.first('trendtable')['office'].split('U.S. ')[-1]
+        chamber = Chamber(name)
+
+        parties = soup.findAll('party')
+
+        for party in parties:
+            party_name = party['title'].lower()
+            won_total = party.first(attrs={'name':'Won'})['value']
+            won_total = int(won_total)
+            setattr(chamber, '%s_won_total' % party_name, won_total)
+
+            current_total = party.first(attrs={'name':'Current'})['value']
+            current_total = int(current_total)
+            setattr(chamber, '%s_current_total' % party_name, current_total)
+
+            holdovers = party.first(attrs={'name':'Holdovers'})['value']
+            holdovers = int(holdovers)
+            setattr(chamber, '%s_holdovers' % party_name, holdovers)
+
+            insuff = party.first(attrs={'name':'InsufficientVote'})['value']
+            insuff = int(insuff)
+            setattr(chamber, '%s_insufficient' % party_name, insuff)
+
+            leading = party.first(attrs={'name':'Leading'})['value']
+            leading = int(leading)
+            setattr(chamber, '%s_leading' % party_name, leading)
+
+            nc_node = party.first('netchange')
+            net_change = nc_node.first(attrs={'name':'Winners'})['value']
+            net_change = int(net_change)
+            setattr(chamber, '%s_net_change' % party_name, net_change)
+        return chamber
+
+    def __str__(self):
+        return self.__unicode__().encode("utf-8")
+     
+    def __repr__(self):
+        return '<%s>' % self.__class__.__name__
+
+    #
+    # public
+    #
+
+    def get_trends(self):
+        """
+        Downloads House & Senate trends files. Re-loads
+        self.chambers with new chamber objects with new
+        net changes for each party.
+        """
+        # Warning: Thar be XML here.
+        remote_dir = '/Trend/xml/'
+        files = ['h.xml', 's.xml']
+        
+        self.chambers = []
+        for f in files:
+            data = self.client._fetch(os.path.join(remote_dir, f))
+            chamber = self._process_trends(data, f)
+            self.chambers.append(chamber)
+        return self.chambers
+
+    @property
+    def house(self):
+        return [c for c in self.chambers if c.name == "House"][0]
+
+    @property
+    def senate(self):
+        return [c for c in self.chambers if c.name == "Senate"][0]
+
+
 class DelegateSummary(object):
     """
     A result collection that contains a summary of delegates won by candidates
@@ -874,6 +1358,22 @@ class Race(object):
     def __repr__(self):
         return '<%s: %s>' % (self.__class__.__name__, self.__unicode__())
     
+    @property
+    def is_referendum(self):
+        """
+        Returns true if this is a race where the people vote to decide about
+        a law, amendment, whatever.
+        """
+        return self.office_name in [
+            'Amendment',
+            'Initiative',
+            'Issue',
+            'Measure',
+            'Proposition',
+            'Question',
+            'Referendum',
+        ]
+    
     @property
     def name(self):
         name = ''
@@ -1020,7 +1520,8 @@ class ReportingUnit(object):
     """
     def __init__(self, ap_number=None, name=None, abbrev=None, fips=None,
                  precincts_total=None, num_reg_voters=None, votes_cast=None,
-                 precincts_reporting=None, precincts_reporting_percent=None):
+                 precincts_reporting=None, precincts_reporting_percent=None,
+                 electoral_votes_total=None):
         self.ap_number = ap_number
         self.name = name
         self.abbrev = abbrev
@@ -1030,6 +1531,7 @@ class ReportingUnit(object):
         self.precincts_total = precincts_total
         self.precincts_reporting = precincts_reporting
         self.precincts_reporting_percent = precincts_reporting_percent
+        self.electoral_votes_total = electoral_votes_total
         self._results = {}
     
     def __unicode__(self):
@@ -1043,9 +1545,13 @@ class ReportingUnit(object):
     
     def __repr__(self):
         return '<%s: %s>' % (self.__class__.__name__, self.__unicode__())
-
+    
     @property
     def key(self):
+        """
+        A combined ID from two AP attributes that we *think* will be unique
+        across states.
+        """
         return "%(name)s%(ap_number)s" % self.__dict__
     
     @property
@@ -1080,7 +1586,7 @@ class Candidate(object):
                  abbrev_name=None, suffix=None, use_suffix=False, 
                  ap_natl_number=None, ap_polra_number=None, ap_race_number=None,
                  party=None, ap_pol_number=None, is_winner=None,
-                 is_runoff=None, delegates=None, is_incumbent=None):
+                 is_runoff=None, delegates=None, is_incumbent=None, vote_total=None):
         self.first_name = first_name
         self.middle_name = middle_name
         self.last_name = last_name
@@ -1096,6 +1602,7 @@ class Candidate(object):
         self.is_runoff = is_runoff
         self.delegates = delegates
         self.is_incumbent = is_incumbent
+        self.vote_total = vote_total
     
     def __unicode__(self):
         return unicode(self.name)
@@ -1120,11 +1627,12 @@ class Result(object):
     The vote count for a candidate in a race in a particular reporting unit.
     """
     def __init__(self, candidate=None, reporting_unit=None, vote_total=None,
-                 vote_total_percent=None):
+                 vote_total_percent=None, electoral_votes_total=None):
         self.candidate = candidate
         self.reporting_unit = reporting_unit
         self.vote_total = vote_total
         self.vote_total_percent = vote_total_percent
+        self.electoral_votes_total = electoral_votes_total
     
     def __unicode__(self):
         return u'%s, %s, %s' % (self.candidate, self.reporting_unit,
@@ -1136,6 +1644,83 @@ class Result(object):
     def __repr__(self):
         return '<%s: %s>' % (self.__class__.__name__, self.__unicode__())
 
+
+#
+# Trend objects
+#
+
+class Chamber(object):
+    """
+    A chamber of Congress. Holds the makeup during an election for 
+    each party in a specific house.
+    """
+    def __init__(self, name):
+        self.name = name
+        self.dem_net_change = None
+        self.gop_net_change = None
+        self.others_net_change = None
+        ##
+        self.dem_won_total = None
+        self.gop_won_total = None
+        self.others_won_total = None
+        ##
+        self.dem_leading = None
+        self.gop_leading = None
+        self.others_leading = None
+        ##
+        self.dem_current_total = None
+        self.gop_current_total = None
+        self.others_current_total = None
+        ##
+        self.dem_holdovers = None
+        self.gop_holdovers = None
+        self.others_holdovers = None
+        ##
+        self.dem_insufficient = None
+        self.gop_insufficient = None
+        self.others_insufficient = None
+
+    @property
+    def gop_uncalled(self):
+        """
+        Uncalled races for GOP.
+        Uncalled = "leading" + insufficient votes
+        """
+        return self.gop_leading + self.gop_insufficient
+
+    @property
+    def dem_uncalled(self):
+        """
+        Uncalled races for Dem.
+        Uncalled = "leading" + insufficient votes
+        """
+        return self.dem_leading + self.dem_insufficient
+
+    @property
+    def others_uncalled(self):
+        """
+        Uncalled races for others.
+        Uncalled = "leading" + insufficient votes
+        """
+        return self.others_leading + self.others_insufficient
+
+    @property
+    def all_uncalled(self):
+        """
+        Uncalled races for all parties.
+        Uncalled = "leading" + insufficient votes
+        """
+        return self.others_uncalled + self.gop_uncalled + self.dem_uncalled
+
+    def __unicode__(self):
+        return unicode(self.name)
+    
+    def __str__(self):
+        return self.__unicode__().encode("utf-8")
+     
+    def __repr__(self):
+        return '<%s: %s>' % (self.__class__.__name__, self.__unicode__())
+
 #
 # Errors
 #
